@@ -23,6 +23,7 @@
 #include <KAboutData>
 #include <KCrash>
 #include <KDBusService>
+#include <KQuickAddons/QtQuickSettings>
 #include <KLocalizedString>
 #include <QCommandLineParser>
 #include <qwindow.h>
@@ -50,12 +51,12 @@ QCommandLineParser* createParser()
     parser->addOption(QCommandLineOption(QStringLiteral("compact"), i18n("Compact Mode (auto/compact/full)."), QStringLiteral("mode"), QStringLiteral("auto")));
     parser->addOption(QCommandLineOption(QStringLiteral("local-filename"), i18n("Local package file to install"), QStringLiteral("package")));
     parser->addOption(QCommandLineOption(QStringLiteral("listbackends"), i18n("List all the available backends.")));
+    parser->addOption(QCommandLineOption(QStringLiteral("search"), i18n("Search string."), QStringLiteral("text")));
+    parser->addOption(QCommandLineOption(QStringLiteral("feedback"), i18n("Lists the available options for user feedback")));
     parser->addOption(QCommandLineOption(QStringLiteral("test"), QStringLiteral("Test file"), QStringLiteral("file.qml")));
     parser->addPositionalArgument(QStringLiteral("urls"), i18n("Supports appstream: url scheme"));
     DiscoverBackendsFactory::setupCommandLine(parser);
     KAboutData::applicationData().setupCommandLine(parser);
-    parser->addHelpOption();
-    parser->addVersionOption();
     return parser;
 }
 
@@ -70,6 +71,11 @@ void processArgs(QCommandLineParser* parser, DiscoverObject* mainWindow)
 
     if(parser->isSet(QStringLiteral("mode")))
         mainWindow->openMode(parser->value(QStringLiteral("mode")));
+    else
+        mainWindow->openMode(QStringLiteral("Browsing"));
+
+    if(parser->isSet(QStringLiteral("search")))
+        Q_EMIT mainWindow->openSearch(parser->value(QStringLiteral("search")));
 
     if(parser->isSet(QStringLiteral("local-filename")))
         mainWindow->openLocalPackage(QUrl::fromUserInput(parser->value(QStringLiteral("local-filename")), {}, QUrl::AssumeLocalFile));
@@ -91,13 +97,23 @@ int main(int argc, char** argv)
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("plasmadiscover")));
     app.setAttribute(Qt::AA_DontCreateNativeWidgetSiblings);
     app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 13, 0)
+    app.setAttribute(Qt::AA_DisableSessionManager, true);
+#endif
     KCrash::initialize();
+    KQuickAddons::QtQuickSettings::init();
     KLocalizedString::setApplicationDomain("plasma-discover");
     KAboutData about(QStringLiteral("discover"), i18n("Discover"), version, i18n("An application explorer"),
-                     KAboutLicense::GPL, i18n("© 2010-2018 Plasma Development Team"));
-    about.addAuthor(i18n("Aleix Pol Gonzalez"), QString(), QStringLiteral("aleixpol@blue-systems.com"));
+                     KAboutLicense::GPL, i18n("© 2010-2019 Plasma Development Team"));
+    about.addAuthor(i18n("Aleix Pol Gonzalez"), QString(), QStringLiteral("aleixpol@kde.org"));
     about.addAuthor(i18n("Jonathan Thomas"), QString(), QStringLiteral("echidnaman@kubuntu.org"));
     about.setProductName("discover/discover");
+    about.setProgramLogo(app.windowIcon());
+
+    about.setTranslator(
+            i18ndc(nullptr, "NAME OF TRANSLATORS", "Your names"),
+            i18ndc(nullptr, "EMAIL OF TRANSLATORS", "Your emails"));
+
     KAboutData::setApplicationData(about);
 
     DiscoverObject *mainWindow = nullptr;
@@ -121,7 +137,15 @@ int main(int argc, char** argv)
 
         KDBusService* service = new KDBusService(KDBusService::Unique, &app);
 
-        mainWindow = new DiscoverObject(s_decodeCompactMode->value(parser->value(QStringLiteral("compact")), DiscoverObject::Full));
+        {
+            auto options = parser->optionNames();
+            options.removeAll(QStringLiteral("backends"));
+            options.removeAll(QStringLiteral("test"));
+            QVariantMap initialProperties;
+            if (!options.isEmpty() || !parser->positionalArguments().isEmpty())
+                initialProperties = {{QStringLiteral("currentTopLevel"), QStringLiteral("qrc:/qml/LoadingPage.qml")}};
+            mainWindow = new DiscoverObject(s_decodeCompactMode->value(parser->value(QStringLiteral("compact")), DiscoverObject::Full), initialProperties);
+        }
         QObject::connect(&app, &QCoreApplication::aboutToQuit, mainWindow, &DiscoverObject::deleteLater);
         QObject::connect(service, &KDBusService::activateRequested, mainWindow, [mainWindow](const QStringList &arguments, const QString &/*workingDirectory*/){
             if (!mainWindow->rootObject())
@@ -145,6 +169,12 @@ int main(int argc, char** argv)
             return 0;
         }
 
+        if(parser->isSet(QStringLiteral("feedback"))) {
+            QTextStream(stdout) << mainWindow->describeSources() << '\n';
+            delete mainWindow;
+            return 0;
+        }
+
         if (parser->isSet(QStringLiteral("test"))) {
             const QUrl testFile = QUrl::fromUserInput(parser->value(QStringLiteral("test")), {}, QUrl::AssumeLocalFile);
             Q_ASSERT(!testFile.isEmpty() && testFile.isLocalFile());
