@@ -23,7 +23,7 @@
 
 #include "TransactionModel.h"
 #include <QMetaProperty>
-#include <QDebug>
+#include "libdiscover_debug.h"
 
 TransactionListener::TransactionListener(QObject *parent)
     : QObject(parent)
@@ -127,6 +127,10 @@ void TransactionListener::setTransaction(Transaction* trans)
         connect(m_transaction, &Transaction::cancellableChanged, this, &TransactionListener::cancellableChanged);
         connect(m_transaction, &Transaction::statusChanged, this, &TransactionListener::transactionStatusChanged);
         connect(m_transaction, &Transaction::progressChanged, this, &TransactionListener::progressChanged);
+        connect(m_transaction, &QObject::destroyed, this, [this]() {
+            qCDebug(LIBDISCOVER_LOG) << "destroyed transaction before finishing";
+            setTransaction(nullptr);
+        });
         setResourceInternal(trans->resource());
     }
     Q_EMIT transactionChanged(trans);
@@ -139,6 +143,7 @@ void TransactionListener::transactionStatusChanged(Transaction::Status status)
         setTransaction(nullptr);
         emit cancelled();
         break;
+    case Transaction::DoneWithErrorStatus:
     case Transaction::DoneStatus:
         setTransaction(nullptr);
         break;
