@@ -172,7 +172,7 @@ void UpdateModel::checkResources(const QList<AbstractResource*>& resource, bool
 Qt::ItemFlags UpdateModel::flags(const QModelIndex &index) const
 {
     if (!index.isValid())
-        return nullptr;
+        return Qt::NoItemFlags;
 
     return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
@@ -318,7 +318,7 @@ UpdateItem * UpdateModel::itemFromResource(AbstractResource* res)
 
 QString UpdateModel::updateSize() const
 {
-    return KFormat().formatByteSize(m_updates->updateSize());
+    return m_updates ? KFormat().formatByteSize(m_updates->updateSize()) : QString();
 }
 
 QModelIndex UpdateModel::indexFromItem(UpdateItem* item) const
