@@ -22,7 +22,7 @@
 #include "DiscoverBackendsFactory.h"
 #include <resources/ResourcesUpdatesModel.h>
 #include <UpdateModel/UpdateModel.h>
-#include <tests/modeltest.h>
+#include <QAbstractItemModelTester>
 #include <resources/ResourcesModel.h>
 #include <resources/ResourcesProxyModel.h>
 #include <resources/AbstractBackendUpdater.h>
@@ -81,7 +81,7 @@ void DummyTest::testReadData()
 {
     const auto resources = fetchResources(m_appBackend->search({}));
 
-    QCOMPARE(m_appBackend->property("startElements").toInt()*2, resources.size());
+    QCOMPARE(m_appBackend->property("startElements").toInt() * 2, resources.size());
     QBENCHMARK {
         for(AbstractResource* res: resources) {
             QVERIFY(!res->name().isEmpty());
@@ -102,18 +102,18 @@ void DummyTest::testProxy()
     QVERIFY(spy.wait());
     QVERIFY(!pm.isBusy());
 
-    QCOMPARE(m_appBackend->property("startElements").toInt()*2, pm.rowCount());
+    QCOMPARE(pm.rowCount(), m_appBackend->property("startElements").toInt() * 2);
     pm.setSearch(QStringLiteral("techie"));
     QVERIFY(pm.isBusy());
     QVERIFY(spy.wait());
     QVERIFY(!pm.isBusy());
-    QCOMPARE(m_appBackend->property("startElements").toInt(), pm.rowCount());
+    QCOMPARE(0, pm.rowCount());
     QCOMPARE(pm.subcategories().count(), 7);
     pm.setSearch(QString());
     QVERIFY(pm.isBusy());
     QVERIFY(spy.wait());
     QVERIFY(!pm.isBusy());
-    QCOMPARE(m_appBackend->property("startElements").toInt()*2, pm.rowCount());
+    QCOMPARE(pm.rowCount(), m_appBackend->property("startElements").toInt() * 2);
 }
 
 void DummyTest::testProxySorting()
@@ -131,7 +131,7 @@ void DummyTest::testProxySorting()
     QVERIFY(spy.wait());
     QVERIFY(!pm.isBusy());
 
-    QCOMPARE(m_appBackend->property("startElements").toInt()*2, pm.rowCount());
+    QCOMPARE(m_appBackend->property("startElements").toInt() * 2, pm.rowCount());
     QVariant lastRatingCount;
     for(int i=0, rc=pm.rowCount(); i<rc; ++i) {
         const QModelIndex mi = pm.index(i, 0);
@@ -149,7 +149,7 @@ void DummyTest::testFetch()
 
     //fetches updates, adds new things
     m_appBackend->checkForUpdates();
-    QSignalSpy spy(m_model, SIGNAL(allInitialized()));
+    QSignalSpy spy(m_model, &ResourcesModel::allInitialized);
     QVERIFY(spy.wait(80000));
     auto resources2 = fetchResources(m_appBackend->search({}));
     QCOMPARE(m_appBackend->property("startElements").toInt()*4, resources2.count());
@@ -187,13 +187,16 @@ void DummyTest::testSort()
 
 void DummyTest::testInstallAddons()
 {
-    const auto resources = fetchResources(m_appBackend->findResourceByPackageName(QUrl(QStringLiteral("dummy://Dummy.1"))));
+    AbstractResourcesBackend::Filters filter;
+    filter.resourceUrl = QUrl(QStringLiteral("dummy://Dummy.1"));
+
+    const auto resources = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources.count(), 1);
     AbstractResource* res = resources.first();
     QVERIFY(res);
 
     ApplicationAddonsModel m;
-    new ModelTest(&m, &m);
+    new QAbstractItemModelTester(&m, &m);
     m.setApplication(res);
     QCOMPARE(m.rowCount(), res->addonsInformation().count());
     QCOMPARE(res->addonsInformation().at(0).isInstalled(), false);
@@ -224,13 +227,16 @@ void DummyTest::testInstallAddons()
 
 void DummyTest::testReviewsModel()
 {
-    const auto resources = fetchResources(m_appBackend->findResourceByPackageName(QUrl(QStringLiteral("dummy://Dummy.1"))));
+    AbstractResourcesBackend::Filters filter;
+    filter.resourceUrl = QUrl(QStringLiteral("dummy://Dummy.1"));
+
+    const auto resources = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources.count(), 1);
     AbstractResource* res = resources.first();
     QVERIFY(res);
 
     ReviewsModel m;
-    new ModelTest(&m, &m);
+    new QAbstractItemModelTester(&m, &m);
     m.setResource(res);
     m.fetchMore();
 
@@ -242,7 +248,7 @@ void DummyTest::testReviewsModel()
     m.markUseful(0, false);
     QCOMPARE(ReviewsModel::UserChoice(m.data(m.index(0,0), ReviewsModel::UsefulChoice).toInt()), ReviewsModel::No);
 
-    const auto resources2 = fetchResources(m_appBackend->findResourceByPackageName(QUrl(QStringLiteral("dummy://Dummy.1"))));
+    const auto resources2 = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources2.count(), 1);
     res = resources2.first();
     m.setResource(res);
@@ -257,21 +263,24 @@ void DummyTest::testUpdateModel()
     const auto backend = m_model->backends().first();
 
     ResourcesUpdatesModel ruModel;
-    new ModelTest(&ruModel, &ruModel);
+    new QAbstractItemModelTester(&ruModel, &ruModel);
     UpdateModel model;
-    new ModelTest(&model, &model);
+    new QAbstractItemModelTester(&model, &model);
     model.setBackend(&ruModel);
 
-    QCOMPARE(model.rowCount(), 4*backend->property("startElements").toInt()/3);
+    QCOMPARE(model.rowCount(), backend->property("startElements").toInt()*2);
     QCOMPARE(model.hasUpdates(), true);
 }
 
 void DummyTest::testScreenshotsModel()
 {
-    ScreenshotsModel m;
-    new ModelTest(&m, &m);
+    AbstractResourcesBackend::Filters filter;
+    filter.resourceUrl = QUrl(QStringLiteral("dummy://Dummy.1"));
 
-    const auto resources = fetchResources(m_appBackend->findResourceByPackageName(QUrl(QStringLiteral("dummy://Dummy.1"))));
+    ScreenshotsModel m;
+    new QAbstractItemModelTester(&m, &m);
+
+    const auto resources = fetchResources(m_appBackend->search(filter));
     QCOMPARE(resources.count(), 1);
     AbstractResource* res = resources.first();
     QVERIFY(res);
