@@ -19,16 +19,16 @@
  ***************************************************************************/
 
 #include "DummyTest.h"
-#include <tests/modeltest.h>
+#include <QAbstractItemModelTester>
 #include <KFormat>
 #include <resources/ResourcesModel.h>
 #include <resources/ResourcesProxyModel.h>
 #include <resources/AbstractBackendUpdater.h>
 #include <ApplicationAddonsModel.h>
-#include <Transaction/TransactionModel.h>
 #include <ReviewsBackend/ReviewsModel.h>
 #include <UpdateModel/UpdateModel.h>
 #include <resources/ResourcesUpdatesModel.h>
+#include <Transaction/TransactionModel.h>
 
 #include <qtest.h>
 #include <QtTest>
@@ -53,8 +53,6 @@ public:
     UpdateDummyTest(QObject* parent = nullptr): QObject(parent)
     {
         m_model = new ResourcesModel(QStringLiteral("dummy-backend"), this);
-        new ModelTest(m_model, m_model);
-
         m_appBackend = backendByName(m_model, QStringLiteral("DummyBackend"));
     }
 
@@ -68,17 +66,43 @@ private Q_SLOTS:
         }
     }
 
-    void testUpdate()
+    void testInformation()
     {
         ResourcesUpdatesModel* rum = new ResourcesUpdatesModel(this);
-        new ModelTest(rum, rum);
+        new QAbstractItemModelTester(rum, rum);
 
         UpdateModel* m = new UpdateModel(this);
-        new ModelTest(m, m);
+        new QAbstractItemModelTester(m, m);
         m->setBackend(rum);
 
         rum->prepare();
-        QCOMPARE(m_appBackend->updatesCount(), m_appBackend->property("startElements").toInt()*2/3);
+        QSignalSpy spySetup(m_appBackend->backendUpdater(), &AbstractBackendUpdater::progressingChanged);
+        QVERIFY(!m_appBackend->backendUpdater()->isProgressing() || spySetup.wait());
+        QCOMPARE(m_appBackend->updatesCount(), m_appBackend->property("startElements").toInt());
+        QCOMPARE(m->hasUpdates(), true);
+
+        QCOMPARE(m->index(0,0).data(UpdateModel::ChangelogRole).toString(), QString{});
+
+        QSignalSpy spy(m, &QAbstractItemModel::dataChanged);
+        m->fetchUpdateDetails(0);
+        QVERIFY(spy.count() || spy.wait());
+        QCOMPARE(spy.count(), 1);
+        delete m;
+    }
+
+    void testUpdate()
+    {
+        ResourcesUpdatesModel* rum = new ResourcesUpdatesModel(this);
+        new QAbstractItemModelTester(rum, rum);
+
+        UpdateModel* m = new UpdateModel(this);
+        new QAbstractItemModelTester(m, m);
+        m->setBackend(rum);
+
+        rum->prepare();
+        QSignalSpy spySetup(m_appBackend->backendUpdater(), &AbstractBackendUpdater::progressingChanged);
+        QVERIFY(!m_appBackend->backendUpdater()->isProgressing() || spySetup.wait());
+        QCOMPARE(m_appBackend->updatesCount(), m_appBackend->property("startElements").toInt());
         QCOMPARE(m->hasUpdates(), true);
 
         for(int i=0, c=m->rowCount(); i<c; ++i) {
@@ -87,7 +111,6 @@ private Q_SLOTS:
 
             AbstractResource* res = qobject_cast<AbstractResource*>(resourceIdx.data(UpdateModel::ResourceRole).value<QObject*>());
             QVERIFY(res);
-            QCOMPARE(resourceIdx.data(UpdateModel::SizeRole).toString(), QStringLiteral("123 B"));
 
             QCOMPARE(Qt::CheckState(resourceIdx.data(Qt::CheckStateRole).toInt()), Qt::Checked);
             QVERIFY(m->setData(resourceIdx, int(Qt::Unchecked), Qt::CheckStateRole));
@@ -107,8 +130,29 @@ private Q_SLOTS:
         QCOMPARE(m->hasUpdates(), true);
 
         rum->prepare();
+
+        spy.clear();
+        QCOMPARE(rum->isProgressing(), false);
         rum->updateAll();
-        QVERIFY(spy.wait());
+        QVERIFY(spy.count() || spy.wait());
+        QCOMPARE(rum->isProgressing(), true);
+
+        QCOMPARE(TransactionModel::global()->rowCount(), 1);
+        connect(TransactionModel::global(), &TransactionModel::progressChanged, this, []() {
+            const int progress = TransactionModel::global()->progress();
+            static int lastProgress = -1;
+            Q_ASSERT(progress >= lastProgress || (TransactionModel::global()->rowCount() == 0 && progress == 0));
+            lastProgress = progress;
+        });
+
+        QTest::qWait(20);
+        QScopedPointer<ResourcesUpdatesModel> rum2(new ResourcesUpdatesModel(this));
+        new QAbstractItemModelTester(rum2.data(), rum2.data());
+
+        QScopedPointer<UpdateModel> m2(new UpdateModel(this));
+        new QAbstractItemModelTester(m2.data(), m2.data());
+        m->setBackend(rum2.data());
+
         QCOMPARE(rum->isProgressing(), true);
         QVERIFY(spy.wait());
         QCOMPARE(rum->isProgressing(), false);
