@@ -25,11 +25,13 @@
 #include "FlatpakSourcesBackend.h"
 #include "FlatpakJobTransaction.h"
 
+#include <utils.h>
 #include <resources/StandardBackendUpdater.h>
 #include <resources/SourcesModel.h>
 #include <Transaction/Transaction.h>
 #include <appstream/OdrsReviewsBackend.h>
 #include <appstream/AppStreamIntegration.h>
+#include <appstream/AppStreamUtils.h>
 
 #include <AppStreamQt/bundle.h>
 #include <AppStreamQt/component.h>
@@ -58,16 +60,27 @@
 
 #include <glib.h>
 #include <QRegularExpression>
-#include "FlatpakSourcesBackend.h"
+
+#include <sys/stat.h>
 
 DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
 
-static FlatpakResource::Id idForInstalledRef(FlatpakInstallation *installation, FlatpakInstalledRef *ref)
+QDebug operator<<(QDebug debug, const FlatpakResource::Id& id)
+{
+    QDebugStateSaver saver(debug);
+    debug.nospace() << "FlatpakResource::Id(";
+    debug.nospace() << "name:" << id.id << ',';
+    debug.nospace() << "branch:" << id.branch << ',';
+    debug.nospace() << "origin:" << id.origin << ',';
+    debug.nospace() << "type:" << id.type;
+    debug.nospace() << ')';
+    return debug;
+}
+
+static FlatpakResource::Id idForInstalledRef(FlatpakInstallation *installation, FlatpakInstalledRef *ref, const QString &postfix)
 {
     const FlatpakResource::ResourceType appType = flatpak_ref_get_kind(FLATPAK_REF(ref)) == FLATPAK_REF_KIND_APP ? FlatpakResource::DesktopApp : FlatpakResource::Runtime;
-    const QString name = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref)));
-    const QString appId = appType == FlatpakResource::DesktopApp ? QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref))) + QStringLiteral(".desktop") : name;
-
+    const QString appId = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref))) + postfix;
     const QString arch = QString::fromUtf8(flatpak_ref_get_arch(FLATPAK_REF(ref)));
     const QString branch = QString::fromUtf8(flatpak_ref_get_branch(FLATPAK_REF(ref)));
 
@@ -96,7 +109,16 @@ FlatpakBackend::FlatpakBackend(QObject* parent)
         SourcesModel::global()->addSourcesBackend(m_sources);
     }
 
-    connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, &FlatpakBackend::announceRatingsReady);
+    connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, [this] {
+        m_reviews->emitRatingFetched(this, kTransform<QList<AbstractResource*>>(m_resources.values(), [] (AbstractResource* r) { return r; }));
+    });
+
+    /* Override the umask to 022 to make it possible to share files between
+     * the plasma-discover process and flatpak system helper process.
+     *
+     * See https://github.com/flatpak/flatpak/pull/2856/
+     */
+    umask(022);
 }
 
 FlatpakBackend::~FlatpakBackend()
@@ -104,7 +126,7 @@ FlatpakBackend::~FlatpakBackend()
     g_cancellable_cancel(m_cancellable);
     m_threadPool.waitForDone(200);
     m_threadPool.clear();
-    for(auto inst : m_installations)
+    for(auto inst : qAsConst(m_installations))
         g_object_unref(inst);
 
     g_object_unref(m_cancellable);
@@ -115,18 +137,6 @@ bool FlatpakBackend::isValid() const
     return m_sources && !m_installations.isEmpty();
 }
 
-void FlatpakBackend::announceRatingsReady()
-{
-    emitRatingsReady();
-
-    const auto ids = m_reviews->appstreamIds().toSet();
-    foreach(AbstractResource* res, m_resources) {
-        if (ids.contains(res->appstreamId())) {
-            res->ratingFetched();
-        }
-    }
-}
-
 class FlatpakFetchRemoteResourceJob : public QNetworkAccessManager
 {
 Q_OBJECT
@@ -140,9 +150,11 @@ public:
 
     void start()
     {
-        auto replyGet = get(QNetworkRequest(m_url));
-
+        QNetworkRequest req(m_url);
+        req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
+        auto replyGet = get(req);
         connect(replyGet, &QNetworkReply::finished, this, [this, replyGet] {
+            QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(replyGet);
             const QUrl originalUrl = replyGet->request().url();
             if (replyGet->error() != QNetworkReply::NoError) {
                 qWarning() << "couldn't download" << originalUrl << replyGet->errorString();
@@ -153,21 +165,35 @@ public:
             const QUrl fileUrl = QUrl::fromLocalFile(QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1Char('/') + originalUrl.fileName());
             auto replyPut = put(QNetworkRequest(fileUrl), replyGet->readAll());
             connect(replyPut, &QNetworkReply::finished, this, [this, originalUrl, fileUrl, replyPut]() {
-                if (replyPut->error() == QNetworkReply::NoError) {
-                    auto res = m_backend->resourceForFile(fileUrl);
-                    if (res) {
-                        FlatpakResource *resource = qobject_cast<FlatpakResource*>(res);
-                        resource->setResourceFile(originalUrl);
-                        Q_EMIT jobFinished(true, resource);
-                    } else {
-                        qWarning() << "couldn't create resource from" << fileUrl.toLocalFile();
-                        Q_EMIT jobFinished(false, nullptr);
-                    }
-                } else {
+                QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(replyPut);
+                if (replyPut->error() != QNetworkReply::NoError) {
                     qWarning() << "couldn't save" << originalUrl << replyPut->errorString();
                     Q_EMIT jobFinished(false, nullptr);
+                    return;
                 }
-            });
+                if (!fileUrl.isLocalFile()) {
+                    Q_EMIT jobFinished(false, nullptr);
+                    return;
+                }
+
+                FlatpakResource *resource = nullptr;
+                if (fileUrl.path().endsWith(QLatin1String(".flatpak"))) {
+                    resource = m_backend->addAppFromFlatpakBundle(fileUrl);
+                } else if (fileUrl.path().endsWith(QLatin1String(".flatpakref"))) {
+                    resource = m_backend->addAppFromFlatpakRef(fileUrl);
+                } else if (fileUrl.path().endsWith(QLatin1String(".flatpakrepo"))) {
+                    resource = m_backend->addSourceFromFlatpakRepo(fileUrl);
+                }
+
+                if (resource) {
+                    resource->setResourceFile(originalUrl);
+                    Q_EMIT jobFinished(true, resource);
+                } else {
+                    qWarning() << "couldn't create resource from" << fileUrl.toLocalFile();
+                    Q_EMIT jobFinished(false, nullptr);
+                }
+            }
+            );
         });
     }
 
@@ -175,8 +201,8 @@ Q_SIGNALS:
     void jobFinished(bool success, FlatpakResource *resource);
 
 private:
-    FlatpakBackend *m_backend;
-    QUrl m_url;
+    FlatpakBackend  *const m_backend;
+    const QUrl m_url;
 };
 
 FlatpakRemote * FlatpakBackend::getFlatpakRemoteByUrl(const QString &url, FlatpakInstallation *installation) const
@@ -206,7 +232,7 @@ FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation
         return ref;
     }
 
-    const auto type = resource->type() == FlatpakResource::DesktopApp ? FLATPAK_REF_KIND_APP : FLATPAK_REF_KIND_RUNTIME;
+    const auto type = resource->resourceType() == FlatpakResource::DesktopApp ? FLATPAK_REF_KIND_APP : FLATPAK_REF_KIND_RUNTIME;
 
     return flatpak_installation_get_installed_ref(flatpakInstallation,
                                                  type,
@@ -218,20 +244,28 @@ FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation
 
 FlatpakResource * FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const
 {
-    return m_resources.value(idForInstalledRef(flatpakInstallation, ref));
+    auto r = m_resources.value(idForInstalledRef(flatpakInstallation, ref, {}));
+    if (!r)
+        r = m_resources.value(idForInstalledRef(flatpakInstallation, ref, QStringLiteral(".desktop")));
+
+//     if (!r) {
+//         qDebug() << "no" << flatpak_ref_get_name(FLATPAK_REF(ref));
+//     }
+    return r;
 }
 
 FlatpakResource * FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) const
 {
     FlatpakResource *runtime = nullptr;
-    const auto runtimeInfo = resource->runtime().split(QLatin1Char('/'));
+    const QString runtimeName = resource->runtime();
+    const auto runtimeInfo = runtimeName.splitRef(QLatin1Char('/'));
 
     if (runtimeInfo.count() != 3) {
         return runtime;
     }
 
     for(auto it = m_resources.constBegin(), itEnd = m_resources.constEnd(); it!=itEnd; ++it) {
-        const auto id = it.key();
+        const auto& id = it.key();
         if (id.type == FlatpakResource::Runtime && id.id == runtimeInfo.at(0) && id.branch == runtimeInfo.at(2)) {
             runtime = *it;
             break;
@@ -371,6 +405,17 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
 {
     QSettings settings(url.toLocalFile(), QSettings::NativeFormat);
     const QString refurl = settings.value(QStringLiteral("Flatpak Ref/Url")).toString();
+    const QString name = settings.value(QStringLiteral("Flatpak Ref/Name")).toString();
+
+    auto item = m_sources->sourceByUrl(refurl);
+    if (item) {
+        const auto resources = resourcesByAppstreamName(name);
+        for (auto resource : resources) {
+            if (resource->origin() == item->data(AbstractSourcesBackend::IdRole)) {
+                return static_cast<FlatpakResource*>(resource);
+            }
+        }
+    }
 
     g_autoptr(GError) error = nullptr;
     g_autoptr(FlatpakRemoteRef) remoteRef = nullptr;
@@ -387,7 +432,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
         remoteRef = flatpak_installation_install_ref_file (preferredInstallation(), bytes, m_cancellable, &error);
         if (!remoteRef) {
             qWarning() << "Failed to create install ref file:" << error->message;
-            const auto resources = resourcesByAppstreamName(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+            const auto resources = resourcesByAppstreamName(name);
             if (!resources.isEmpty()) {
                 return qobject_cast<FlatpakResource*>(resources.constFirst());
             }
@@ -404,7 +449,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
     asComponent.setDescription(settings.value(QStringLiteral("Flatpak Ref/Description")).toString());
     asComponent.setName(settings.value(QStringLiteral("Flatpak Ref/Title")).toString());
     asComponent.setSummary(settings.value(QStringLiteral("Flatpak Ref/Comment")).toString());
-    asComponent.setId(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+    asComponent.setId(name);
 
     const QString iconUrl = settings.value(QStringLiteral("Flatpak Ref/Icon")).toString();
     if (!iconUrl.isEmpty()) {
@@ -470,7 +515,7 @@ FlatpakResource * FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url)
         return nullptr;
     }
 
-    if (gpgKey.startsWith(QStringLiteral("http://")) || gpgKey.startsWith(QStringLiteral("https://"))) {
+    if (gpgKey.startsWith(QLatin1String("http://")) || gpgKey.startsWith(QLatin1String("https://"))) {
         return nullptr;
     }
 
@@ -521,6 +566,10 @@ void FlatpakBackend::addResource(FlatpakResource *resource)
     updateAppSize(installation, resource);
 
     m_resources.insert(resource->uniqueId(), resource);
+    if (!resource->extends().isEmpty()) {
+        m_extends.append(resource->extends());
+        m_extends.removeDuplicates();
+    }
 }
 
 class FlatpakSource
@@ -556,6 +605,9 @@ void FlatpakBackend::loadAppsFromAppstreamData()
 {
     for (auto installation : qAsConst(m_installations)) {
         // Load applications from appstream metadata
+        if (g_cancellable_is_cancelled(m_cancellable))
+            break;
+
         if (!loadAppsFromAppstreamData(installation)) {
             qWarning() << "Failed to load packages from appstream data from installation" << installation;
         }
@@ -575,7 +627,7 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
 
     for (uint i = 0; i < remotes->len; i++) {
         FlatpakRemote *remote = FLATPAK_REMOTE(g_ptr_array_index(remotes, i));
-        g_autoptr(GFile) fileTimestamp = flatpak_remote_get_appstream_timestamp(remote, nullptr);
+        g_autoptr(GFile) fileTimestamp = flatpak_remote_get_appstream_timestamp(remote, flatpak_get_default_arch());
 
         QFileInfo fileInfo = QFileInfo(QString::fromUtf8(g_file_get_path(fileTimestamp)));
         // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
@@ -585,17 +637,25 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
             integrateRemote(flatpakInstallation, remote);
         }
     }
-
     return true;
 }
 
+void FlatpakBackend::metadataRefreshed()
+{
+    m_refreshAppstreamMetadataJobs--;
+    if (m_refreshAppstreamMetadataJobs == 0) {
+        loadInstalledApps();
+        checkForUpdates();
+    }
+}
+
 void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
 {
     Q_ASSERT(m_refreshAppstreamMetadataJobs != 0);
-    m_refreshAppstreamMetadataJobs--;
 
     FlatpakSource source(remote);
     if (!source.isEnabled() || flatpak_remote_get_noenumerate(remote)) {
+        metadataRefreshed();
         return;
     }
 
@@ -604,6 +664,7 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
     const QString appDirFileName = appstreamDirPath + QLatin1String("/appstream.xml.gz");
     if (!QFile::exists(appDirFileName)) {
         qWarning() << "No" << appDirFileName << "appstream metadata found for" << source.name();
+        metadataRefreshed();
         return;
     }
 
@@ -617,11 +678,8 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
             resource->setOrigin(sourceName);
             addResource(resource);
         }
-        if (!m_refreshAppstreamMetadataJobs) {
-            loadInstalledApps();
-            checkForUpdates();
 
-        }
+        metadataRefreshed();
         acquireFetching(false);
         fw->deleteLater();
     });
@@ -666,27 +724,32 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
     for (uint i = 0; i < refs->len; i++) {
         FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
 
-        if (flatpak_ref_get_kind(FLATPAK_REF(ref)) == FLATPAK_REF_KIND_RUNTIME) {
+        const auto name = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref)));
+        if (name.endsWith(QLatin1String(".Debug")) || name.endsWith(QLatin1String(".Locale")) || name.endsWith(QLatin1String(".BaseApp")) || name.endsWith(QLatin1String(".Docs")))
             continue;
-        }
+
         const auto res = getAppForInstalledRef(flatpakInstallation, ref);
         if (res) {
             res->setState(AbstractResource::Installed);
             continue;
         }
 
-        const auto name = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref)));
+        AppStream::Component cid;
         AppStream::Metadata metadata;
         const QString fnDesktop = pathApps + name + QLatin1String(".desktop");
         AppStream::Metadata::MetadataError error = metadata.parseFile(fnDesktop, AppStream::Metadata::FormatKindDesktopEntry);
         if (error != AppStream::Metadata::MetadataErrorNoError) {
-            qWarning() << "Failed to parse appstream metadata: " << error << fnDesktop;
-            continue;
-        }
+            if (QFile::exists(fnDesktop))
+                qDebug() << "Failed to parse appstream metadata:" << error << fnDesktop;
 
-        auto component = metadata.component();
-        component.setId(name + QLatin1String(".desktop"));
-        FlatpakResource *resource = new FlatpakResource(component, flatpakInstallation, this);
+            cid.setId(QString::fromLatin1(flatpak_ref_get_name(FLATPAK_REF(ref))));
+#if FLATPAK_CHECK_VERSION(1,1,2)
+            cid.setName(QString::fromLatin1(flatpak_installed_ref_get_appdata_name(ref)));
+#endif
+        } else
+            cid = metadata.component();
+
+        FlatpakResource *resource = new FlatpakResource(cid, flatpakInstallation, this);
 
         resource->setIconPath(pathExports);
         resource->setState(AbstractResource::Installed);
@@ -737,12 +800,18 @@ void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation* installation)
 {
     auto fw = new QFutureWatcher<GPtrArray *>(this);
     connect(fw, &QFutureWatcher<GPtrArray *>::finished, this, [this, installation, fw](){
-        auto refs = fw->result();
+        g_autoptr(GPtrArray) refs = fw->result();
         onFetchUpdatesFinished(installation, refs);
         fw->deleteLater();
+        acquireFetching(false);
     });
+    acquireFetching(true);
     fw->setFuture(QtConcurrent::run(&m_threadPool, [installation, this]() -> GPtrArray * {
         g_autoptr(GError) localError = nullptr;
+        if (g_cancellable_is_cancelled(m_cancellable)) {
+            qWarning() << "don't issue commands after cancelling";
+            return {};
+        }
         GPtrArray *refs = flatpak_installation_list_installed_refs_for_update(installation, m_cancellable, &localError);
         if (!refs) {
             qWarning() << "Failed to get list of installed refs for listing updates: " << localError->message;
@@ -751,22 +820,21 @@ void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation* installation)
     }));
 }
 
-void FlatpakBackend::onFetchUpdatesFinished(FlatpakInstallation *flatpakInstallation, GPtrArray *updates)
+void FlatpakBackend::onFetchUpdatesFinished(FlatpakInstallation *flatpakInstallation, GPtrArray *fetchedUpdates)
 {
-    if (!updates) {
+    if (!fetchedUpdates) {
         qWarning() << "could not get updates for" << flatpakInstallation;
         return;
     }
 
-    g_autoptr(GPtrArray) fetchedUpdates = updates;
-
     for (uint i = 0; i < fetchedUpdates->len; i++) {
         FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(fetchedUpdates, i));
         FlatpakResource *resource = getAppForInstalledRef(flatpakInstallation, ref);
         if (resource) {
             resource->setState(AbstractResource::Upgradeable);
             updateAppSize(flatpakInstallation, resource);
-        }
+        } else
+            qWarning() << "could not find updated resource" << flatpak_ref_get_name(FLATPAK_REF(ref)) << m_resources.size();
     }
 }
 
@@ -824,15 +892,16 @@ public:
 #else
         if (!flatpak_installation_update_appstream_sync(m_installation, flatpak_remote_get_name(m_remote), nullptr, nullptr, m_cancellable, &localError)) {
 #endif
-            qWarning() << "Failed to refresh appstream metadata for " << flatpak_remote_get_name(m_remote) << ": " << (localError ? localError->message : "<no error>");
-            Q_EMIT jobRefreshAppstreamMetadataFailed();
+            const QString error = localError ? QString::fromUtf8(localError->message) : QStringLiteral("<no error>");
+            qWarning() << "Failed to refresh appstream metadata for " << flatpak_remote_get_name(m_remote) << ": " << error;
+            Q_EMIT jobRefreshAppstreamMetadataFailed(error);
         } else  {
             Q_EMIT jobRefreshAppstreamMetadataFinished(m_installation, m_remote);
         }
     }
 
 Q_SIGNALS:
-    void jobRefreshAppstreamMetadataFailed();
+    void jobRefreshAppstreamMetadataFailed(const QString &errorMessage);
     void jobRefreshAppstreamMetadataFinished(FlatpakInstallation *installation, FlatpakRemote *remote);
 
 private:
@@ -844,10 +913,12 @@ private:
 void FlatpakBackend::refreshAppstreamMetadata(FlatpakInstallation *installation, FlatpakRemote *remote)
 {
     FlatpakRefreshAppstreamMetadataJob *job = new FlatpakRefreshAppstreamMetadataJob(installation, remote);
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, [this] () {
-        m_refreshAppstreamMetadataJobs--;
-    });
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, &FlatpakBackend::metadataRefreshed);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, [this] (const QString &errorMessage) { Q_EMIT passiveMessage(errorMessage); });
     connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFinished, this, &FlatpakBackend::integrateRemote);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::finished, this, [this] { acquireFetching(false); });
+
+    acquireFetching(true);
     job->start();
 }
 
@@ -889,14 +960,11 @@ void FlatpakBackend::updateAppInstalledMetadata(FlatpakInstalledRef *installedRe
 
 bool FlatpakBackend::updateAppMetadata(FlatpakInstallation* flatpakInstallation, FlatpakResource *resource)
 {
-    g_autoptr(GFile) installationPath = nullptr;
-
-    if (resource->type() != FlatpakResource::DesktopApp) {
+    if (resource->resourceType() != FlatpakResource::DesktopApp) {
         return true;
     }
 
-    installationPath = flatpak_installation_get_path(flatpakInstallation);
-    const QString path = QString::fromUtf8(g_file_get_path(installationPath)) + QStringLiteral("/app/%1/%2/%3/active/metadata").arg(resource->flatpakName()).arg(resource->arch()).arg(resource->branch());
+    const QString path = resource->installPath() + QStringLiteral("/metadata");
 
     if (QFile::exists(path)) {
         return updateAppMetadata(resource, path);
@@ -921,7 +989,7 @@ void FlatpakBackend::onFetchMetadataFinished(FlatpakInstallation *flatpakInstall
 
     // Right now we attempt to update metadata for calculating the size so call updateSizeFromRemote()
     // as it's what we want. In future if there are other reason to update metadata we will need to somehow
-    // distinguish betwen these calls
+    // distinguish between these calls
     updateAppSizeFromRemote(flatpakInstallation, resource);
 }
 
@@ -951,7 +1019,7 @@ bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource, const QByteArr
 
 bool FlatpakBackend::updateAppSize(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
 {
-    // Check if the size is already set, we should also distiguish between download and installed size,
+    // Check if the size is already set, we should also distinguish between download and installed size,
     // right now it doesn't matter whether we get size for installed or not installed app, but if we
     // start making difference then for not installed app check download and install size separately
 
@@ -979,7 +1047,7 @@ bool FlatpakBackend::updateAppSize(FlatpakInstallation *flatpakInstallation, Fla
 bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
 {
     // Calculate the runtime size
-    if (resource->state() == AbstractResource::None && resource->type() == FlatpakResource::DesktopApp) {
+    if (resource->state() == AbstractResource::None && resource->resourceType() == FlatpakResource::DesktopApp) {
         auto runtime = getRuntimeForApp(resource);
         if (runtime) {
             // Re-check runtime state if case a new one was created
@@ -1031,7 +1099,7 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
 void FlatpakBackend::onFetchSizeFinished(FlatpakResource *resource, guint64 downloadSize, guint64 installedSize)
 {
     FlatpakResource *runtime = nullptr;
-    if (resource->state() == AbstractResource::None && resource->type() == FlatpakResource::DesktopApp) {
+    if (resource->state() == AbstractResource::None && resource->resourceType() == FlatpakResource::DesktopApp) {
         runtime = getRuntimeForApp(resource);
     }
 
@@ -1050,7 +1118,7 @@ void FlatpakBackend::updateAppState(FlatpakInstallation *flatpakInstallation, Fl
         // If the app is installed, we can set information about commit, arch etc.
         updateAppInstalledMetadata(ref, resource);
     } else {
-        // TODO check if the app is actuall still available
+        // TODO check if the app is actually still available
         resource->setState(AbstractResource::None);
     }
 }
@@ -1084,13 +1152,13 @@ bool FlatpakBackend::flatpakResourceLessThan(AbstractResource* l, AbstractResour
 
 ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
-    if (filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakrepo")) || filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakref"))) {
-        auto stream = new ResultsStream(QStringLiteral("FlatpakStream-http-")+filter.resourceUrl.fileName());
+    if (filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakrepo")) || filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakref")) || filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpak"))) {
+        auto stream = new ResultsStream(QLatin1String("FlatpakStream-http-")+filter.resourceUrl.fileName());
 
         FlatpakFetchRemoteResourceJob *fetchResourceJob = new FlatpakFetchRemoteResourceJob(filter.resourceUrl, this);
         connect(fetchResourceJob, &FlatpakFetchRemoteResourceJob::jobFinished, this, [fetchResourceJob, stream] (bool success, FlatpakResource *resource) {
             if (success) {
-                stream->resourcesFound({resource});
+                Q_EMIT stream->resourcesFound({resource});
             }
             stream->finish();
             fetchResourceJob->deleteLater();
@@ -1100,21 +1168,25 @@ ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &
         return stream;
     } else if(filter.resourceUrl.scheme() == QLatin1String("appstream")) {
         return findResourceByPackageName(filter.resourceUrl);
-    } else if (!filter.resourceUrl.isEmpty() || !filter.extends.isEmpty())
+    } else if (!filter.resourceUrl.isEmpty() || (!filter.extends.isEmpty() && !m_extends.contains(filter.extends)))
         return new ResultsStream(QStringLiteral("FlatpakStream-void"), {});
 
     auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
     auto f = [this, stream, filter] () {
         QVector<AbstractResource*> ret;
-        foreach(AbstractResource* r, m_resources) {
-            if (r->isTechnical() && filter.state != AbstractResource::Upgradeable) {
+        for (auto r : qAsConst(m_resources)) {
+            const bool matchById = r->appstreamId().compare(filter.search, Qt::CaseInsensitive) == 0;
+            if (r->type() == AbstractResource::Technical && filter.state != AbstractResource::Upgradeable && !matchById) {
                 continue;
             }
-
             if (r->state() < filter.state)
                 continue;
 
-            if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive)) {
+            if (!filter.extends.isEmpty() && !r->extends().contains(filter.extends))
+                continue;
+
+            if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive) || matchById)
+            {
                 ret += r;
             }
         }
@@ -1147,12 +1219,13 @@ QVector<AbstractResource *> FlatpakBackend::resourcesByAppstreamName(const QStri
 ResultsStream * FlatpakBackend::findResourceByPackageName(const QUrl &url)
 {
     if (url.scheme() == QLatin1String("appstream")) {
-        if (url.host().isEmpty())
-            passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
+        const auto appstreamIds = AppStreamUtils::appstreamIds(url);
+        if (appstreamIds.isEmpty())
+            Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
         else {
             auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
-            auto f = [this, stream, url] () {
-                const auto resources = resourcesByAppstreamName(url.host());
+            auto f = [this, stream, appstreamIds] () {
+                const auto resources = kAppend<QVector<AbstractResource*>>(appstreamIds, [this] (const QString appstreamId) { return resourcesByAppstreamName(appstreamId); });
                 if (!resources.isEmpty())
                     Q_EMIT stream->resourcesFound(resources);
                 stream->finish();
@@ -1185,7 +1258,7 @@ Transaction* FlatpakBackend::installApplication(AbstractResource *app, const Add
 
     FlatpakResource *resource = qobject_cast<FlatpakResource*>(app);
 
-    if (resource->type() == FlatpakResource::Source) {
+    if (resource->resourceType() == FlatpakResource::Source) {
         // Let source backend handle this
         FlatpakRemote *remote = m_sources->installSource(resource);
         if (remote) {
@@ -1198,35 +1271,10 @@ Transaction* FlatpakBackend::installApplication(AbstractResource *app, const Add
         return nullptr;
     }
 
-    FlatpakJobTransaction *transaction = nullptr;
-    FlatpakInstallation *installation = resource->installation();
-
-    if (resource->propertyState(FlatpakResource::RequiredRuntime) == FlatpakResource::NotKnownYet && resource->type() == FlatpakResource::DesktopApp) {
-        transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole, true);
-        connect(resource, &FlatpakResource::propertyStateChanged, [resource, transaction, this] (FlatpakResource::PropertyKind kind, FlatpakResource::PropertyState state) {
-            if (kind != FlatpakResource::RequiredRuntime) {
-                return;
-            }
-
-            if (state == FlatpakResource::AlreadyKnown) {
-                FlatpakResource *runtime = getRuntimeForApp(resource);
-                if (runtime && !runtime->isInstalled()) {
-                    transaction->setRuntime(runtime);
-                }
-            }
-            transaction->start();
-        });
-    } else {
-        FlatpakResource *runtime = getRuntimeForApp(resource);
-        if (runtime && !runtime->isInstalled()) {
-            transaction = new FlatpakJobTransaction(resource, runtime, Transaction::InstallRole);
-        } else {
-            transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole);
-        }
-    }
-
-    connect(transaction, &FlatpakJobTransaction::statusChanged, [this, installation, resource] (Transaction::Status status) {
+    FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole);
+    connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
+            FlatpakInstallation *installation = resource->installation();
             updateAppState(installation, resource);
         }
     });
@@ -1242,7 +1290,7 @@ Transaction* FlatpakBackend::removeApplication(AbstractResource *app)
 {
     FlatpakResource *resource = qobject_cast<FlatpakResource*>(app);
 
-    if (resource->type() == FlatpakResource::Source) {
+    if (resource->resourceType() == FlatpakResource::Source) {
         // Let source backend handle this
         if (m_sources->removeSource(resource->flatpakName())) {
             resource->setState(AbstractResource::None);
@@ -1253,7 +1301,7 @@ Transaction* FlatpakBackend::removeApplication(AbstractResource *app)
     FlatpakInstallation *installation = resource->installation();
     FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::RemoveRole);
 
-    connect(transaction, &FlatpakJobTransaction::statusChanged, [this, installation, resource] (Transaction::Status status) {
+    connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, installation, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
             updateAppSize(installation, resource);
         }
@@ -1267,27 +1315,15 @@ void FlatpakBackend::checkForUpdates()
         // Load local updates, comparing current and latest commit
         loadLocalUpdates(installation);
 
+        if (g_cancellable_is_cancelled(m_cancellable))
+            break;
+
         // Load updates from remote repositories
         loadRemoteUpdates(installation);
-    }
-}
 
-AbstractResource * FlatpakBackend::resourceForFile(const QUrl &url)
-{
-    if (!url.isLocalFile()) {
-        return nullptr;
+        if (g_cancellable_is_cancelled(m_cancellable))
+            break;
     }
-
-    FlatpakResource *resource = nullptr;
-    if (url.path().endsWith(QLatin1String(".flatpak"))) {
-        resource = addAppFromFlatpakBundle(url);
-    } else if (url.path().endsWith(QLatin1String(".flatpakref"))) {
-        resource = addAppFromFlatpakRef(url);
-    } else if (url.path().endsWith(QLatin1String(".flatpakrepo"))) {
-        resource = addSourceFromFlatpakRepo(url);
-    }
-
-    return resource;
 }
 
 QString FlatpakBackend::displayName() const
