@@ -35,7 +35,6 @@ extern "C" {
 #include <flatpak.h>
 }
 
-class QAction;
 class FlatpakSourcesBackend;
 class StandardBackendUpdater;
 class OdrsReviewsBackend;
@@ -58,11 +57,17 @@ public:
     Transaction* installApplication(AbstractResource* app, const AddonList& addons) override;
     Transaction* removeApplication(AbstractResource* app) override;
     bool isFetching() const override { return m_isFetching>0; }
-    AbstractResource * resourceForFile(const QUrl & ) override;
     void checkForUpdates() override;
     QString displayName() const override;
     bool hasApplications() const override { return true; }
     FlatpakResource * addSourceFromFlatpakRepo(const QUrl &url);
+    QStringList extends() const override { return m_extends; }
+
+    FlatpakResource * addAppFromFlatpakBundle(const QUrl &url);
+    FlatpakResource * addAppFromFlatpakRef(const QUrl &url);
+    FlatpakResource * getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const;
+
+    FlatpakSourcesBackend *sources() const { return m_sources; }
 
 private Q_SLOTS:
     void onFetchMetadataFinished(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, const QByteArray &metadata);
@@ -73,17 +78,15 @@ Q_SIGNALS: //for tests
     void initialized();
 
 private:
+    void metadataRefreshed();
     bool flatpakResourceLessThan(AbstractResource* l, AbstractResource* r) const;
     void announceRatingsReady();
     FlatpakInstallation * preferredInstallation() const { return m_installations.constFirst(); }
     void integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote);
     FlatpakRemote * getFlatpakRemoteByUrl(const QString &url, FlatpakInstallation *installation) const;
     FlatpakInstalledRef * getInstalledRefForApp(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource) const;
-    FlatpakResource * getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const;
     FlatpakResource * getRuntimeForApp(FlatpakResource *resource) const;
 
-    FlatpakResource * addAppFromFlatpakBundle(const QUrl &url);
-    FlatpakResource * addAppFromFlatpakRef(const QUrl &url);
     void addResource(FlatpakResource *resource);
     void loadAppsFromAppstreamData();
     bool loadAppsFromAppstreamData(FlatpakInstallation *flatpakInstallation);
@@ -111,6 +114,7 @@ private:
     QSharedPointer<OdrsReviewsBackend> m_reviews;
     uint m_isFetching = 0;
     uint m_refreshAppstreamMetadataJobs;
+    QStringList m_extends;
 
     GCancellable *m_cancellable;
     QVector<FlatpakInstallation *> m_installations;
