@@ -21,6 +21,8 @@
 
 #include "FlatpakResource.h"
 #include "FlatpakBackend.h"
+#include "FlatpakSourcesBackend.h"
+#include "config-paths.h"
 
 #include <Transaction/AddonList.h>
 
@@ -31,6 +33,7 @@
 #include <KFormat>
 #include <KLocalizedString>
 
+#include <QProcess>
 #include <QDir>
 #include <QDebug>
 #include <QDesktopServices>
@@ -46,20 +49,19 @@
 static QString iconCachePath(const AppStream::Icon &icon)
 {
     Q_ASSERT(icon.kind() == AppStream::Icon::KindRemote);
-    return QStringLiteral("%1/icons/%2").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)).arg(icon.url().fileName());
+    return QStringLiteral("%1/icons/%2").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation), icon.url().fileName());
 }
 
 FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakInstallation* installation, FlatpakBackend *parent)
     : AbstractResource(parent)
     , m_appdata(component)
+    , m_id({installation, QString(), FlatpakResource::DesktopApp, component.id(), QString(), QString() })
     , m_downloadSize(0)
     , m_installedSize(0)
     , m_propertyStates({{DownloadSize, NotKnownYet}, {InstalledSize, NotKnownYet},{RequiredRuntime, NotKnownYet}})
-    , m_installation(installation)
     , m_state(AbstractResource::None)
-    , m_type(FlatpakResource::DesktopApp)
 {
-    setObjectName(component.id());
+    setObjectName(packageName());
 
     // Start fetching remote icons during initialization
     const auto icons = m_appdata.icons();
@@ -72,7 +74,7 @@ FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakI
                     // Create $HOME/.cache/discover/icons folder
                     cacheDir.mkdir(QStringLiteral("icons"));
                     QNetworkAccessManager *manager = new QNetworkAccessManager(this);
-                    connect(manager, &QNetworkAccessManager::finished, [this, icon, fileName, manager] (QNetworkReply *reply) {
+                    connect(manager, &QNetworkAccessManager::finished, this, [this, icon, fileName, manager] (QNetworkReply *reply) {
                         if (reply->error() == QNetworkReply::NoError) {
                             QByteArray iconData = reply->readAll();
                             QFile file(fileName);
@@ -98,7 +100,7 @@ AppStream::Component FlatpakResource::appstreamComponent() const
 
 QList<PackageState> FlatpakResource::addonsInformation()
 {
-    return m_addons;
+    return {};
 }
 
 QString FlatpakResource::availableVersion() const
@@ -118,22 +120,22 @@ QString FlatpakResource::availableVersion() const
 
 QString FlatpakResource::appstreamId() const
 {
-    return m_appdata.id();
+    return m_id.id;
 }
 
 QString FlatpakResource::arch() const
 {
-    return m_arch;
+    return m_id.arch;
 }
 
 QString FlatpakResource::branch() const
 {
-    return m_branch;
+    return m_id.branch;
 }
 
 bool FlatpakResource::canExecute() const
 {
-    return (m_type == DesktopApp && (m_state == AbstractResource::Installed || m_state == AbstractResource::Upgradeable));
+    return (m_id.type == DesktopApp && (m_state == AbstractResource::Installed || m_state == AbstractResource::Upgradeable));
 }
 
 void FlatpakResource::updateFromRef(FlatpakRef* ref)
@@ -142,7 +144,15 @@ void FlatpakResource::updateFromRef(FlatpakRef* ref)
     setBranch(QString::fromUtf8(flatpak_ref_get_branch(ref)));
     setCommit(QString::fromUtf8(flatpak_ref_get_commit(ref)));
     setFlatpakName(QString::fromUtf8(flatpak_ref_get_name(ref)));
-    setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? FlatpakResource::DesktopApp : FlatpakResource::Runtime);
+    setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? DesktopApp :
+                                          extends().isEmpty() ? Runtime
+                                                              : Extension);
+    setObjectName(packageName());
+}
+
+QString FlatpakResource::ref() const
+{
+    return typeAsString() + QLatin1Char('/') + flatpakName() + QLatin1Char('/') + arch() + QLatin1Char('/') + branch();
 }
 
 QStringList FlatpakResource::categories()
@@ -183,8 +193,6 @@ QVariant FlatpakResource::icon() const
     } else if (icons.isEmpty()) {
         ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
     } else foreach(const AppStream::Icon &icon, icons) {
-        QStringList stock;
-
         switch (icon.kind()) {
             case AppStream::Icon::KindLocal:
             case AppStream::Icon::KindCached: {
@@ -198,9 +206,12 @@ QVariant FlatpakResource::icon() const
                     }
                 }
             }   break;
-            case AppStream::Icon::KindStock:
-                stock += icon.name();
+            case AppStream::Icon::KindStock: {
+                const auto ret = QIcon::fromTheme(icon.name());
+                if (!ret.isNull())
+                    return ret;
                 break;
+            }
             case AppStream::Icon::KindRemote: {
                 const QString fileName = iconCachePath(icon);
                 if (QFileInfo::exists(fileName)) {
@@ -211,11 +222,12 @@ QVariant FlatpakResource::icon() const
             case AppStream::Icon::KindUnknown:
                 break;
         }
-
-        if (ret.isNull()) {
-            ret = QIcon::fromTheme(stock.value(0), QIcon::fromTheme(QStringLiteral("package-x-generic")));
-        }
     }
+
+    if (ret.isNull()) {
+        ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
+    }
+
     return ret;
 }
 
@@ -235,9 +247,16 @@ int FlatpakResource::installedSize() const
     return m_installedSize;
 }
 
-bool FlatpakResource::isTechnical() const
+AbstractResource::Type FlatpakResource::type() const
 {
-    return m_type == FlatpakResource::Runtime;
+    switch (m_id.type) {
+        case FlatpakResource::Runtime:
+            return Technical;
+        case FlatpakResource::Extension:
+            return Addon;
+        default:
+            return Application;
+    }
 }
 
 QUrl FlatpakResource::homepage()
@@ -245,6 +264,21 @@ QUrl FlatpakResource::homepage()
     return m_appdata.url(AppStream::Component::UrlKindHomepage);
 }
 
+QUrl FlatpakResource::helpURL()
+{
+    return m_appdata.url(AppStream::Component::UrlKindHelp);
+}
+
+QUrl FlatpakResource::bugURL()
+{
+    return m_appdata.url(AppStream::Component::UrlKindBugtracker);
+}
+
+QUrl FlatpakResource::donationURL()
+{
+    return m_appdata.url(AppStream::Component::UrlKindDonation);
+}
+
 QString FlatpakResource::flatpakFileType() const
 {
     return m_flatpakFileType;
@@ -255,15 +289,15 @@ QString FlatpakResource::flatpakName() const
     // If the flatpak name is not known (known only for installed apps), then use
     // appstream id instead;
     if (m_flatpakName.isEmpty()) {
-        return m_appdata.id();
+        return m_id.id;
     }
 
     return m_flatpakName;
 }
 
-QString FlatpakResource::license()
+QJsonArray FlatpakResource::licenses()
 {
-    return m_appdata.projectLicense();
+    return AppStreamUtils::licenses(m_appdata);
 }
 
 QString FlatpakResource::longDescription()
@@ -275,7 +309,7 @@ QString FlatpakResource::name() const
 {
     QString name = m_appdata.name();
     if (name.isEmpty()) {
-        name = m_appdata.id();
+        name = flatpakName();
     }
 
     if (name.startsWith(QLatin1String("(Nightly) "))) {
@@ -287,7 +321,7 @@ QString FlatpakResource::name() const
 
 QString FlatpakResource::origin() const
 {
-    return m_origin;
+    return m_id.origin;
 }
 
 QString FlatpakResource::packageName() const
@@ -346,43 +380,47 @@ QString FlatpakResource::sizeDescription()
     }
 }
 
-AbstractResource::State FlatpakResource::state() const
+AbstractResource::State FlatpakResource::state()
 {
     return m_state;
 }
 
-FlatpakResource::ResourceType FlatpakResource::type() const
+FlatpakResource::ResourceType FlatpakResource::resourceType() const
 {
-    return m_type;
+    return m_id.type;
 }
 
 QString FlatpakResource::typeAsString() const
 {
-    switch (m_type) {
+    switch (m_id.type) {
+        case FlatpakResource::Runtime:
+        case FlatpakResource::Extension:
+            return QLatin1String("runtime");
         case FlatpakResource::DesktopApp:
         case FlatpakResource::Source:
-            return QLatin1String("app");
-        case FlatpakResource::Runtime:
-            return QLatin1String("runtime");
         default:
             return QLatin1String("app");
     }
 }
 
-QString FlatpakResource::uniqueId() const
+FlatpakResource::Id FlatpakResource::uniqueId() const
 {
-    return installationPath() + QStringLiteral("/flatpak/") + origin()
-                              + QLatin1Char('/') + typeAsString()
-                              + QLatin1Char('/') + m_appdata.id()
-                              + QLatin1Char('/') + branch();
+    return m_id;
 }
 
 void FlatpakResource::invokeApplication() const
 {
+    const QString desktopFile = installPath() + QLatin1String("/export/share/applications/") + appstreamId();
+    const QString runservice = QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice");
+    if (QFile::exists(desktopFile) && QFile::exists(runservice)) {
+        QProcess::startDetached(runservice, {desktopFile});
+        return;
+    }
+
     g_autoptr(GCancellable) cancellable = g_cancellable_new();
     g_autoptr(GError) localError = nullptr;
 
-    if (!flatpak_installation_launch(m_installation,
+    if (!flatpak_installation_launch(m_id.installation,
                                      flatpakName().toUtf8().constData(),
                                      arch().toUtf8().constData(),
                                      branch().toUtf8().constData(),
@@ -400,29 +438,18 @@ void FlatpakResource::fetchChangelog()
 
 void FlatpakResource::fetchScreenshots()
 {
-    QList<QUrl> thumbnails, screenshots;
-
-    Q_FOREACH (const AppStream::Screenshot &s, m_appdata.screenshots()) {
-        const QUrl thumbnail = AppStreamUtils::imageOfKind(s.images(), AppStream::Image::KindThumbnail);
-        const QUrl plain = AppStreamUtils::imageOfKind(s.images(), AppStream::Image::KindSource);
-        if (plain.isEmpty())
-            qWarning() << "invalid screenshot for" << name();
-
-        screenshots << plain;
-        thumbnails << (thumbnail.isEmpty() ? plain : thumbnail);
-    }
-
-    Q_EMIT screenshotsFetched(thumbnails, screenshots);
+    const auto sc = AppStreamUtils::fetchScreenshots(m_appdata);
+    Q_EMIT screenshotsFetched(sc.first, sc.second);
 }
 
 void FlatpakResource::setArch(const QString &arch)
 {
-    m_arch = arch;
+    m_id.arch = arch;
 }
 
 void FlatpakResource::setBranch(const QString &branch)
 {
-    m_branch = branch;
+    m_id.branch = branch;
 }
 
 void FlatpakResource::setBundledIcon(const QPixmap &pixmap)
@@ -470,7 +497,7 @@ void FlatpakResource::setInstalledSize(int size)
 
 void FlatpakResource::setOrigin(const QString &origin)
 {
-    m_origin = origin;
+    m_id.origin = origin;
 }
 
 void FlatpakResource::setPropertyState(FlatpakResource::PropertyKind kind, FlatpakResource::PropertyState newState)
@@ -500,18 +527,19 @@ void FlatpakResource::setState(AbstractResource::State state)
     if (m_state != state) {
         m_state = state;
 
-        Q_EMIT stateChanged();
+        if (!backend()->isFetching())
+            Q_EMIT stateChanged();
     }
 }
 
 void FlatpakResource::setType(FlatpakResource::ResourceType type)
 {
-    m_type = type;
+    m_id.type = type;
 }
 
 QString FlatpakResource::installationPath() const
 {
-    return installationPath(m_installation);
+    return installationPath(m_id.installation);
 }
 
 QString FlatpakResource::installationPath(FlatpakInstallation* flatpakInstallation)
@@ -520,7 +548,46 @@ QString FlatpakResource::installationPath(FlatpakInstallation* flatpakInstallati
     return QString::fromUtf8(g_file_get_path(path));
 }
 
+QString FlatpakResource::installPath() const
+{
+    return installationPath() + QStringLiteral("/app/%1/%2/%3/active").arg(flatpakName(), arch(), branch());
+}
+
 QUrl FlatpakResource::url() const
 {
     return m_resourceFile.isEmpty() ? QUrl(QStringLiteral("appstream://") + appstreamId()) : m_resourceFile;
 }
+
+QDate FlatpakResource::releaseDate() const
+{
+    if (!m_appdata.releases().isEmpty()) {
+        auto release = m_appdata.releases().constFirst();
+        return release.timestamp().date();
+    }
+
+    return {};
+}
+
+QString FlatpakResource::sourceIcon() const
+{
+    const auto sourceItem = qobject_cast<FlatpakBackend*>(backend())->sources()->sourceById(origin());
+    if (!sourceItem) {
+        qWarning() << "Could not find source " << origin();
+        return QStringLiteral("flatpak-discover");
+    }
+
+    const auto iconUrl = sourceItem->data(FlatpakSourcesBackend::IconUrlRole).toString();
+    if (iconUrl.isEmpty())
+        return QStringLiteral("flatpak-discover");
+    return iconUrl;
+}
+
+    QString FlatpakResource::author() const
+{
+    return m_appdata.developerName();
+}
+
+QStringList FlatpakResource::extends() const
+{
+    return m_appdata.extends();
+}
