@@ -24,6 +24,8 @@
 
 #include <resources/AbstractSourcesBackend.h>
 #include <QStandardItemModel>
+#include <QStack>
+#include <functional>
 
 extern "C" {
 #include <flatpak.h>
@@ -37,11 +39,13 @@ public:
     explicit FlatpakSourcesBackend(const QVector<FlatpakInstallation *>& installations, AbstractResourcesBackend *parent);
     ~FlatpakSourcesBackend() override;
 
+    enum Roles { IconUrlRole = LastRole + 1 };
+
     QAbstractItemModel* sources() override;
     bool addSource(const QString &id) override;
     bool removeSource(const QString &id) override;
     QString idDescription() override;
-    QList<QAction*> actions() const override;
+    QVariantList actions() const override;
     bool supportsAdding() const override { return true; }
     bool canFilterSources() const override { return true; }
 
@@ -50,9 +54,13 @@ public:
 
     bool moveSource(const QString & sourceId, int delta) override;
     int originIndex(const QString& sourceId) const;
+    QStandardItem* sourceByUrl(const QString & url) const;
+    QStandardItem* sourceById(const QString & sourceId) const;
+
+    void cancel() override;
+    void proceed() override;
 
 private:
-    QStandardItem* sourceById(const QString & sourceId) const;
     bool listRepositories(FlatpakInstallation *installation);
     void addRemote(FlatpakRemote *remote, FlatpakInstallation *installation);
 
@@ -60,6 +68,7 @@ private:
     QStandardItemModel* m_sources;
     QAction* const m_flathubAction;
     QStandardItem* m_noSourcesItem;
+    QStack<std::function<void()>> m_proceedFunctions;
 };
 
 #endif // FLATPAKSOURCESBACKEND_H
