@@ -22,12 +22,12 @@
 #define KNSBACKEND_H
 
 #include <KNSCore/EntryInternal>
+#include <KNSCore/ErrorCode>
 
 #include <resources/AbstractResourcesBackend.h>
 #include "Transaction/AddonList.h"
 #include "discovercommon_export.h"
 
-class KConfigGroup;
 class KNSReviews;
 class KNSResource;
 class StandardBackendUpdater;
@@ -52,6 +52,7 @@ public:
     ResultsStream* findResourceByPackageName(const QUrl & search);
 
     QVector<Category*> category() const override { return m_rootCategories; }
+    bool hasApplications() const override { return m_hasApplications; }
 
     bool isValid() const override;
 
@@ -75,6 +76,8 @@ Q_SIGNALS:
 public Q_SLOTS:
     void receivedEntries(const KNSCore::EntryInternal::List& entries);
     void statusChanged(const KNSCore::EntryInternal& entry);
+    void detailsLoaded(const KNSCore::EntryInternal& entry);
+    void signalErrorCode(const KNSCore::ErrorCode& errorCode, const QString& message, const QVariant& metadata);
 
 private:
     void fetchInstalled();
@@ -82,7 +85,8 @@ private:
     void setFetching(bool f);
     void markInvalid(const QString &message);
     void searchStream(ResultsStream* stream, const QString &searchText);
-    
+    void fetchMore();
+
     bool m_onePage = false;
     bool m_responsePending = false;
     bool m_fetching;
@@ -98,6 +102,7 @@ private:
     QVector<Category*> m_rootCategories;
     QString m_displayName;
     bool m_initialized = false;
+    bool m_hasApplications = false;
 };
 
 #endif // KNSBACKEND_H
