@@ -30,6 +30,9 @@
 #include <KConfigGroup>
 #include <KLocalizedString>
 
+#include "libdiscover_backend_debug.h"
+#include "utils.h"
+
 int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage)
 {
     const auto was = percentage;
@@ -38,16 +41,17 @@ int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage
             { PackageKit::Transaction::Status::StatusDownload, 0 },
             { PackageKit::Transaction::Status::StatusInstall, 1},
             { PackageKit::Transaction::Status::StatusRemove, 1},
+            { PackageKit::Transaction::Status::StatusLoadingCache, 1},
             { PackageKit::Transaction::Status::StatusUpdate, 1}
         };
         const auto idx = statuses.value(status, -1);
         if (idx < 0) {
-            qDebug() << "Status not present" << status << "among" << statuses   .keys() << percentage;
+            qCDebug(LIBDISCOVER_BACKEND_LOG) << "Status not present" << status << "among" << statuses   .keys() << percentage;
             return -1;
         }
         percentage = (idx * 100 + percentage) / 2 /*the maximum in statuses*/;
     }
-    qDebug() << "reporting progress with status:" << status << percentage << was;
+    qCDebug(LIBDISCOVER_BACKEND_LOG) << "reporting progress with status:" << status << percentage << was;
     return percentage;
 }
 
@@ -84,10 +88,11 @@ void PackageKitUpdater::prepare()
 void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFlags flags)
 {
     m_packagesModified.clear();
-    auto pkgs = involvedPackages(m_toUpgrade).toList();
+    auto pkgs = involvedPackages(m_toUpgrade).values();
     pkgs.sort();
     m_transaction = PackageKit::Daemon::updatePackages(pkgs, flags);
     m_isCancelable = m_transaction->allowCancel();
+    cancellableChanged();
 
     connect(m_transaction.data(), &PackageKit::Transaction::finished, this, &PackageKitUpdater::finished);
     connect(m_transaction.data(), &PackageKit::Transaction::package, this, &PackageKitUpdater::packageResolved);
@@ -115,7 +120,7 @@ QSet<AbstractResource*> PackageKitUpdater::packagesForPackageId(const QSet<QStri
     QSet<AbstractResource*> ret;
     foreach (AbstractResource * res, m_allUpgradeable) {
         PackageKitResource* pres = qobject_cast<PackageKitResource*>(res);
-        if (packages.contains(pres->allPackageNames().toSet())) {
+        if (packages.contains(kToSet(pres->allPackageNames()))) {
             ret.insert(res);
         }
     }
@@ -196,7 +201,7 @@ void PackageKitUpdater::start()
 
 void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time*/)
 {
-//     qDebug() << "update finished!" << exit << time;
+//     qCDebug(LIBDISCOVER_BACKEND_LOG) << "update finished!" << exit << time;
     if (!m_proceedFunctions.isEmpty())
         return;
     const bool cancel = exit == PackageKit::Transaction::ExitCancelled;
@@ -223,7 +228,7 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
     m_backend->fetchUpdates();
     fetchLastUpdateTime();
 
-    if (useOfflineUpdates()) {
+    if (useOfflineUpdates() && exit == PackageKit::Transaction::ExitSuccess) {
         PackageKit::Daemon::global()->offline()->trigger(PackageKit::Offline::ActionReboot);
         Q_EMIT passiveMessage(i18n("Please restart the computer to finish the update"));
     }
@@ -258,19 +263,24 @@ qreal PackageKitUpdater::progress() const
 
 void PackageKitUpdater::removeResources(const QList<AbstractResource*>& apps)
 {
-    QSet<QString> pkgs = involvedPackages(apps.toSet());
+    QSet<QString> pkgs = involvedPackages(kToSet(apps));
     m_toUpgrade.subtract(packagesForPackageId(pkgs));
 }
 
 void PackageKitUpdater::addResources(const QList<AbstractResource*>& apps)
 {
-    QSet<QString> pkgs = involvedPackages(apps.toSet());
+    QSet<QString> pkgs = involvedPackages(kToSet(apps));
     m_toUpgrade.unite(packagesForPackageId(pkgs));
 }
 
 QList<AbstractResource*> PackageKitUpdater::toUpdate() const
 {
-    return m_toUpgrade.toList();
+    return m_toUpgrade
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    .values();
+#else
+    .toList();
+#endif
 }
 
 bool PackageKitUpdater::isMarked(AbstractResource* res) const
@@ -371,7 +381,7 @@ AbstractBackendUpdater::State toUpdateState(PackageKit::Transaction::Status t)
         case PackageKit::Transaction::StatusCancel:
             return AbstractBackendUpdater::Done;
         default:
-            qDebug() << "unknown packagekit status" << t;
+            qCDebug(LIBDISCOVER_BACKEND_LOG) << "unknown packagekit status" << t;
             return AbstractBackendUpdater::None;
     }
     Q_UNREACHABLE();
