@@ -24,10 +24,11 @@
 #include <resources/AbstractResource.h>
 #include <resources/AbstractResourcesBackend.h>
 #include <QVariantList>
+#include <QVector>
 #include <Snapd/Client>
+#include <functional>
 
-class QAction;
-class SnapReviewsBackend;
+class OdrsReviewsBackend;
 class StandardBackendUpdater;
 class SnapResource;
 class SnapBackend : public AbstractResourcesBackend
@@ -35,6 +36,7 @@ class SnapBackend : public AbstractResourcesBackend
 Q_OBJECT
 public:
     explicit SnapBackend(QObject* parent = nullptr);
+    ~SnapBackend() override;
 
     ResultsStream * search(const AbstractResourcesBackend::Filters & search) override;
     ResultsStream * findResourceByPackageName(const QUrl& search);
@@ -56,12 +58,22 @@ public:
 
 private:
     void setFetching(bool fetching);
+
+    template <class T>
+    ResultsStream* populateWithFilter(T* snaps, std::function<bool(const QSharedPointer<QSnapdSnap>&)>& filter);
+
+    template <class T>
+    ResultsStream* populateJobsWithFilter(const QVector<T*>& snaps, std::function<bool(const QSharedPointer<QSnapdSnap>&)>& filter);
+
     template <class T>
     ResultsStream* populate(T* snaps);
 
+    template <class T>
+    ResultsStream* populate(const QVector<T*>& snaps);
+
     QHash<QString, SnapResource*> m_resources;
     StandardBackendUpdater* m_updater;
-    SnapReviewsBackend* m_reviews;
+    QSharedPointer<OdrsReviewsBackend> m_reviews;
 
     bool m_valid = true;
     bool m_fetching = false;
