@@ -21,13 +21,14 @@
 #ifndef ABSTRACTRESOURCE_H
 #define ABSTRACTRESOURCE_H
 
-#include <QtCore/QObject>
+#include <QObject>
 #include <QUrl>
 #include <QStringList>
 #include <QScopedPointer>
 #include <QVector>
 #include <QCollatorSortKey>
 #include <QJsonObject>
+#include <QJsonArray>
 #include <QDate>
 
 #include "discovercommon_export.h"
@@ -55,15 +56,14 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(State state READ state NOTIFY stateChanged)
     Q_PROPERTY(QString status READ status NOTIFY stateChanged)
     Q_PROPERTY(QStringList category READ categories CONSTANT)
-    Q_PROPERTY(bool isTechnical READ isTechnical CONSTANT)
     Q_PROPERTY(QUrl homepage READ homepage CONSTANT)
     Q_PROPERTY(QUrl helpURL READ helpURL CONSTANT)
     Q_PROPERTY(QUrl bugURL READ bugURL CONSTANT)
     Q_PROPERTY(QUrl donationURL READ donationURL CONSTANT)
     Q_PROPERTY(bool canUpgrade READ canUpgrade NOTIFY stateChanged)
     Q_PROPERTY(bool isInstalled READ isInstalled NOTIFY stateChanged)
-    Q_PROPERTY(QString license READ license CONSTANT)
-    Q_PROPERTY(QString longDescription READ longDescription CONSTANT)
+    Q_PROPERTY(QJsonArray licenses READ licenses CONSTANT)
+    Q_PROPERTY(QString longDescription READ longDescription NOTIFY longDescriptionChanged)
     Q_PROPERTY(QString origin READ origin CONSTANT)
     Q_PROPERTY(QString displayOrigin READ displayOrigin CONSTANT)
     Q_PROPERTY(int size READ size NOTIFY sizeChanged)
@@ -79,6 +79,7 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QUrl url READ url CONSTANT)
     Q_PROPERTY(QString executeLabel READ executeLabel CONSTANT)
     Q_PROPERTY(QString sourceIcon READ sourceIcon CONSTANT)
+    Q_PROPERTY(QString author READ author CONSTANT)
     Q_PROPERTY(QDate releaseDate READ releaseDate NOTIFY stateChanged)
     public:
         /**
@@ -141,19 +142,24 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
         ///@returns a URL that points to the place where you can donate money to the app developer
         virtual QUrl donationURL();
 
-        virtual bool isTechnical() const;
+        enum Type { Application, Addon, Technical };
+        Q_ENUM(Type);
+        virtual Type type() const = 0;
 
         virtual int size() = 0;
         virtual QString sizeDescription();
-        virtual QString license() = 0;
+
+        ///@returns a list of pairs with the name of the license and a URL pointing at it
+        virtual QJsonArray licenses() = 0;
 
         virtual QString installedVersion() const = 0;
         virtual QString availableVersion() const = 0;
         virtual QString longDescription() = 0;
 
         virtual QString origin() const = 0;
-        QString displayOrigin() const;
+        virtual QString displayOrigin() const;
         virtual QString section() = 0;
+        virtual QString author() const = 0;
 
         ///@returns what kind of mime types the resource can consume
         virtual QStringList mimetypes() const;
@@ -170,7 +176,7 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
         bool canUpgrade();
         bool isInstalled();
 
-        ///@returns a user-readable explaination of the resource status
+        ///@returns a user-readable explanation of the resource status
         ///by default, it will specify what state() is returning
         virtual QString status();
 
@@ -219,6 +225,7 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
         void sizeChanged();
         void stateChanged();
         void ratingFetched();
+        void longDescriptionChanged();
 
         ///response to the fetchScreenshots method
         ///@p thumbnails and @p screenshots should have the same number of elements
