@@ -21,9 +21,9 @@
 #ifndef ABSTRACTRESOURCESBACKEND_H
 #define ABSTRACTRESOURCESBACKEND_H
 
-#include <QtCore/QObject>
-#include <QtCore/QPair>
-#include <QtCore/QVector>
+#include <QObject>
+#include <QPair>
+#include <QVector>
 
 #include "AbstractResource.h"
 #include "Transaction/AddonList.h"
@@ -49,6 +49,7 @@ class DISCOVERCOMMON_EXPORT ResultsStream : public QObject
 
     Q_SIGNALS:
         void resourcesFound(const QVector<AbstractResource*>& resources);
+        void fetchMore();
 };
 
 /**
@@ -66,7 +67,7 @@ class DISCOVERCOMMON_EXPORT ResultsStream : public QObject
  * To show resources in Muon, we need to initialize all resources we want to show beforehand,
  * we should not create resources in the search function. When we reload the resources
  * (e.g. when initializing), the backend needs change the fetching property throughout the
- * processs.
+ * process.
  */
 class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
 {
@@ -158,8 +159,6 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
          */
         void emitRatingsReady();
 
-        virtual AbstractResource* resourceForFile(const QUrl &/*url*/) { return nullptr; }
-
         /**
          * @returns the root category tree
          */
@@ -167,6 +166,8 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
 
         virtual bool hasApplications() const { return false; }
 
+        virtual int fetchingUpdatesProgress() const;
+
     public Q_SLOTS:
         /**
          * This gets called when the backend should install an application.
@@ -222,6 +223,7 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
         void resourceRemoved(AbstractResource* resource);
 
         void passiveMessage(const QString &message);
+        void fetchingUpdatesProgressChanged();
 
     private:
         QString m_name;
