@@ -24,8 +24,6 @@
 #include <QDBusInterface>
 #include <QDBusPendingReply>
 #include <QNetworkConfigurationManager>
-#include <KConfig>
-#include <KConfigGroup>
 #include <KRun>
 #include <KLocalizedString>
 #include <KNotification>
@@ -35,10 +33,7 @@
 
 DiscoverNotifier::DiscoverNotifier(QObject * parent)
     : QObject(parent)
-    , m_manager(new QNetworkConfigurationManager(this))
 {
-    configurationChanged();
-
     m_backends = BackendNotifierFactory().allBackends();
     foreach(BackendNotifierModule* module, m_backends) {
         connect(module, &BackendNotifierModule::foundUpdates, this, &DiscoverNotifier::updateStatusNotifier);
@@ -58,30 +53,22 @@ DiscoverNotifier::DiscoverNotifier(QObject * parent)
     m_timer.setInterval(1000);
     updateStatusNotifier();
 
-    connect(m_manager, &QNetworkConfigurationManager::onlineStateChanged, this, &DiscoverNotifier::stateChanged);
-
     //Only fetch updates after the system is comfortably booted
-    QTimer::singleShot(10000, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+    QTimer::singleShot(20000, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
 }
 
 DiscoverNotifier::~DiscoverNotifier() = default;
 
-void DiscoverNotifier::configurationChanged()
-{
-    KConfig notifierConfig(QStringLiteral("plasma-discover-notifierrc"), KConfig::NoGlobals);
-
-    KConfigGroup notifyTypeGroup(&notifierConfig, "NotificationType");
-    m_verbose = notifyTypeGroup.readEntry("Verbose", false);
-}
-
 void DiscoverNotifier::showDiscover()
 {
     KRun::runCommand(QStringLiteral("plasma-discover"), nullptr);
+    if (m_updatesAvailableNotification) { m_updatesAvailableNotification->close(); }
 }
 
 void DiscoverNotifier::showDiscoverUpdates()
 {
     KRun::runCommand(QStringLiteral("plasma-discover --mode update"), nullptr);
+    if (m_updatesAvailableNotification) { m_updatesAvailableNotification->close(); }
 }
 
 void DiscoverNotifier::showUpdatesNotification()
@@ -92,11 +79,11 @@ void DiscoverNotifier::showUpdatesNotification()
         return;
     }
 
-    auto e = KNotification::event(QStringLiteral("Update"), message(), {}, iconName(), nullptr, KNotification::CloseOnTimeout, QStringLiteral("discoverabstractnotifier"));
-    const QString name = i18n("Update");
-    e->setDefaultAction(name);
-    e->setActions({name});
-    connect(e, QOverload<unsigned int>::of(&KNotification::activated), this, &DiscoverNotifier::showDiscoverUpdates);
+    m_updatesAvailableNotification = KNotification::event(QStringLiteral("Update"), message(), {}, iconName(), nullptr, KNotification::CloseOnTimeout, QStringLiteral("discoverabstractnotifier"));
+    const QString name = i18n("View Updates");
+    m_updatesAvailableNotification->setDefaultAction(name);
+    m_updatesAvailableNotification->setActions({name});
+    connect(m_updatesAvailableNotification, QOverload<unsigned int>::of(&KNotification::activated), this, &DiscoverNotifier::showDiscoverUpdates);
 }
 
 void DiscoverNotifier::updateStatusNotifier()
@@ -121,7 +108,7 @@ DiscoverNotifier::State DiscoverNotifier::state() const
 {
     if (m_needsReboot)
         return RebootRequired;
-    else if (!m_manager->isOnline())
+    else if (m_manager && !m_manager->isOnline())
         return Offline;
     else if (m_hasSecurityUpdates)
         return SecurityUpdates;
@@ -167,6 +154,14 @@ QString DiscoverNotifier::message() const
 
 void DiscoverNotifier::recheckSystemUpdateNeeded()
 {
+    if (!m_manager) {
+        m_manager = new QNetworkConfigurationManager(this);
+        connect(m_manager, &QNetworkConfigurationManager::onlineStateChanged, this, &DiscoverNotifier::stateChanged);
+        if (!m_manager->isOnline()) {
+            emit stateChanged();
+        }
+    }
+
     foreach(BackendNotifierModule* module, m_backends)
         module->recheckSystemUpdateNeeded();
 }
@@ -183,7 +178,7 @@ void DiscoverNotifier::showRebootNotification()
 {
     KNotification *notification = new KNotification(QStringLiteral("notification"), KNotification::Persistent | KNotification::DefaultEvent);
     notification->setIconName(QStringLiteral("system-software-update"));
-    notification->setActions(QStringList{QLatin1String("Restart")});
+    notification->setActions(QStringList{i18nc("@action:button", "Restart")});
     notification->setTitle(i18n("Restart is required"));
     notification->setText(i18n("The system needs to be restarted for the updates to take effect."));
 
@@ -195,14 +190,14 @@ void DiscoverNotifier::showRebootNotification()
 void DiscoverNotifier::reboot()
 {
     QDBusInterface interface(QStringLiteral("org.kde.ksmserver"), QStringLiteral("/KSMServer"), QStringLiteral("org.kde.KSMServerInterface"), QDBusConnection::sessionBus());
-    interface.asyncCall(QStringLiteral("logout"), 0, 1, 2); // Options: do not ask again | reboot | force
+    interface.asyncCall(QStringLiteral("logout"), 1, 1, 2); // Options: confirm first | reboot | force
 }
 
 void DiscoverNotifier::foundUpgradeAction(UpgradeAction* action)
 {
     KNotification *notification = new KNotification(QStringLiteral("distupgrade-notification"), KNotification::Persistent | KNotification::DefaultEvent);
     notification->setIconName(QStringLiteral("system-software-update"));
-    notification->setActions(QStringList{QLatin1String("Upgrade")});
+    notification->setActions(QStringList{i18nc("@action:button", "Upgrade")});
     notification->setTitle(i18n("Upgrade available"));
     notification->setText(i18n("New version: %1", action->description()));
 
